@@ -91,13 +91,19 @@ static struct {
   { "header",                 CONFIGSET_SKIN },
   { "footer",                 CONFIGSET_SKIN },
   { "details",                CONFIGSET_SKIN },
+  { "js",                     CONFIGSET_SKIN },
   { "logo-mimetype",          CONFIGSET_SKIN },
   { "logo-image",             CONFIGSET_SKIN },
   { "background-mimetype",    CONFIGSET_SKIN },
   { "background-image",       CONFIGSET_SKIN },
   { "timeline-block-markup",  CONFIGSET_SKIN },
+  { "timeline-date-format",   CONFIGSET_SKIN },
+  { "timeline-dwelltime",     CONFIGSET_SKIN },
+  { "timeline-closetime",     CONFIGSET_SKIN },
   { "timeline-max-comment",   CONFIGSET_SKIN },
   { "timeline-plaintext",     CONFIGSET_SKIN },
+  { "timeline-truncate-at-blank", CONFIGSET_SKIN },
+  { "timeline-utc",           CONFIGSET_SKIN },
   { "adunit",                 CONFIGSET_SKIN },
   { "adunit-omit-if-admin",   CONFIGSET_SKIN },
   { "adunit-omit-if-user",    CONFIGSET_SKIN },
