@@ -316,8 +316,9 @@ char *fossilize(const char *zIn, int nIn){
 */
 void defossilize(char *z){
   int i, j, c;
-  for(i=0; (c=z[i])!=0 && c!='\\'; i++){}
-  if( c==0 ) return;
+  char *zSlash = strchr(z, '\\');
+  if( zSlash==0 ) return;
+  i = zSlash - z;
   for(j=i; (c=z[i])!=0; i++){
     if( c=='\\' && z[i+1] ){
       i++;
@@ -378,9 +379,11 @@ u32 fossil_utf8_read(
 }
 
 /*
-** Encode a UTF8 string for JSON.  All special characters are escaped.
+** Encode a UTF8 string as a JSON string literal (without the surrounding
+** "...") and return a pointer to the encoding.  Space to hold the encoding
+** is obtained from fossil_malloc() and must be freed by the caller.
 */
-void blob_append_json_string(Blob *pBlob, const char *zStr){
+char *encode_json_string_literal(const char *zStr){
   const unsigned char *z;
   char *zOut;
   u32 c;
@@ -400,10 +403,10 @@ void blob_append_json_string(Blob *pBlob, const char *zStr){
       n++;
     }
   }
-  i = blob_size(pBlob);
-  blob_resize(pBlob, i+n);
-  zOut = blob_buffer(pBlob);
+  zOut = fossil_malloc(n+1);
+  if( zOut==0 ) return 0;
   z = (const unsigned char*)zStr;
+  i = 0;
   while( (c = fossil_utf8_read(&z))!=0 ){
     if( c=='\\' ){
       zOut[i++] = '\\';
@@ -427,6 +430,7 @@ void blob_append_json_string(Blob *pBlob, const char *zStr){
     }
   }
   zOut[i] = 0;
+  return zOut;
 }
 
 /*
@@ -645,6 +649,9 @@ int decode16(const unsigned char *zIn, unsigned char *pOut, int N){
 int validate16(const char *zIn, int nIn){
   int i;
   if( nIn<0 ) nIn = (int)strlen(zIn);
+  if( zIn[nIn]==0 ){
+    return strspn(zIn,"0123456789abcdefABCDEF")==nIn;
+  }
   for(i=0; i<nIn; i++, zIn++){
     if( zDecode[zIn[0]&0xff]>63 ){
       return zIn[0]==0;
