@@ -455,6 +455,9 @@ void finfo_page(void){
     hyperlinked_path(zFilename, &title, 0, "tree", "");
     if( fShowId ) blob_appendf(&title, " (%d)", fnid);
   }
+  if( uBg ){
+    blob_append(&title, " (color-coded by user)", -1);
+  }
   @ <h2>%b(&title)</h2>
   blob_reset(&title);
   pGraph = graph_init();
@@ -647,12 +650,13 @@ void finfo_page(void){
   db_finalize(&q);
   db_finalize(&qparent);
   if( pGraph ){
-    graph_finish(pGraph, 1);
+    graph_finish(pGraph, 0, TIMELINE_DISJOINT);
     if( pGraph->nErr ){
       graph_free(pGraph);
       pGraph = 0;
     }else{
-      @ <tr class="timelineBottom"><td></td><td></td><td></td></tr>
+      @ <tr class="timelineBottom" id="btm-%d(iTableId)">\
+      @ <td></td><td></td><td></td></tr>
     }
   }
   @ </table>
@@ -781,7 +785,7 @@ void mlink_page(void){
        mid
     );
     @ <h1>MLINK table for check-in %h(zCI)</h1>
-    render_checkin_context(mid, 1);
+    render_checkin_context(mid, 0, 1);
     style_table_sorter();
     @ <hr />
     @ <div class='brlist'>
