@@ -69,7 +69,7 @@ struct mkd_renderer {
   /* span level callbacks - NULL or return 0 prints the span verbatim */
   int (*autolink)(struct Blob *ob, struct Blob *link,
           enum mkd_autolink type, void *opaque);
-  int (*codespan)(struct Blob *ob, struct Blob *text, void *opaque);
+  int (*codespan)(struct Blob *ob, struct Blob *text, int nSep, void *opaque);
   int (*double_emphasis)(struct Blob *ob, struct Blob *text,
             char c, void *opaque);
   int (*emphasis)(struct Blob *ob, struct Blob *text, char c,void*opaque);
@@ -87,7 +87,6 @@ struct mkd_renderer {
   void (*normal_text)(struct Blob *ob, struct Blob *text, void *opaque);
 
   /* renderer data */
-  int max_work_stack; /* prevent arbitrary deep recursion, cf README */
   const char *emph_chars; /* chars that trigger emphasis rendering */
   void *opaque; /* opaque data send to every rendering callback */
 };
@@ -156,8 +155,9 @@ struct render {
   struct mkd_renderer make;
   struct Blob refs;
   char_trigger active_char[256];
-  int work_active;
-  struct Blob *work;
+  int iDepth;                    /* Depth of recursion */
+  int nBlobCache;                /* Number of entries in aBlobCache */
+  struct Blob *aBlobCache[20];   /* Cache of Blobs available for reuse */
 };
 
 
@@ -240,7 +240,7 @@ static int build_ref_id(struct Blob *id, const char *data, size_t size){
     blob_append(id, data+beg, i-beg);
 
     /* add a single space and skip all consecutive whitespace */
-    if( i<size ) blob_append(id, " ", 1);
+    if( i<size ) blob_append_char(id, ' ');
     while( i<size && (data[i]==' ' || data[i]=='\t' || data[i]=='\n') ){ i++; }
   }
 
@@ -302,25 +302,36 @@ static const struct html_tag *find_block_tag(const char *data, size_t size){
                  cmp_html_tag);
 }
 
+/* return true if recursion has gone too deep */
+static int too_deep(struct render *rndr){
+  return rndr->iDepth>200;
+}
 
-/* new_work_buffer -- get a new working buffer from the stack or create one */
+/* get a new working buffer from the cache or create one.  return NULL
+** if failIfDeep is true and the depth of recursion has gone too deep. */
 static struct Blob *new_work_buffer(struct render *rndr){
-  struct Blob *ret = 0;
-
-  if( rndr->work_active < rndr->make.max_work_stack ){
-    ret = rndr->work + rndr->work_active;
-    rndr->work_active += 1;
-    blob_reset(ret);
+  struct Blob *ret;
+  rndr->iDepth++;
+  if( rndr->nBlobCache ){
+    ret = rndr->aBlobCache[--rndr->nBlobCache];
+  }else{
+    ret = fossil_malloc(sizeof(*ret));
   }
+  *ret = empty_blob;
   return ret;
 }
 
 
-/* release_work_buffer -- release the given working buffer */
+/* release the given working buffer back to the cache */
 static void release_work_buffer(struct render *rndr, struct Blob *buf){
   if( !buf ) return;
-  assert(rndr->work_active>0 && buf==(rndr->work+rndr->work_active-1));
-  rndr->work_active -= 1;
+  rndr->iDepth--;
+  blob_reset(buf);
+  if( rndr->nBlobCache < sizeof(rndr->aBlobCache)/sizeof(rndr->aBlobCache[0]) ){
+    rndr->aBlobCache[rndr->nBlobCache++] = buf;
+  }else{
+    fossil_free(buf);
+  }
 }
 
 
@@ -423,6 +434,10 @@ static void parse_inline(
   char_trigger action = 0;
   struct Blob work = BLOB_INITIALIZER;
 
+  if( too_deep(rndr) ){
+    blob_append(ob, data, size);
+    return;
+  }
   while( i<size ){
     /* copying inactive chars into the output */
     while( end<size
@@ -555,9 +570,9 @@ static size_t parse_emph1(
      && data[i-1]!=' '
      && data[i-1]!='\t'
      && data[i-1]!='\n'
+     && !too_deep(rndr)
     ){
       work = new_work_buffer(rndr);
-      if( !work ) return 0;
       parse_inline(work, rndr, data, i);
       r = rndr->make.emphasis(ob, work, c, rndr->make.opaque);
       release_work_buffer(rndr, work);
@@ -593,9 +608,9 @@ static size_t parse_emph2(
      && data[i-1]!=' '
      && data[i-1]!='\t'
      && data[i-1]!='\n'
+     && !too_deep(rndr)
     ){
       work = new_work_buffer(rndr);
-      if( !work ) return 0;
       parse_inline(work, rndr, data, i);
       r = rndr->make.double_emphasis(ob, work, c, rndr->make.opaque);
       release_work_buffer(rndr, work);
@@ -633,10 +648,10 @@ static size_t parse_emph3(
      && data[i+1]==c
      && data[i+2] == c
      && rndr->make.triple_emphasis
+     && !too_deep(rndr)
     ){
       /* triple symbol found */
       struct Blob *work = new_work_buffer(rndr);
-      if( !work ) return 0;
       parse_inline(work, rndr, data, i);
       r = rndr->make.triple_emphasis(ob, work, c, rndr->make.opaque);
       release_work_buffer(rndr, work);
@@ -727,14 +742,15 @@ static size_t char_codespan(
   size_t size
 ){
   size_t end, nb = 0, i, f_begin, f_end;
+  char delim = data[0];
 
   /* counting the number of backticks in the delimiter */
-  while( nb<size && data[nb]=='`' ){ nb++; }
+  while( nb<size && data[nb]==delim ){ nb++; }
 
   /* finding the next delimiter */
   i = 0;
   for(end=nb; end<size && i<nb; end++){
-    if( data[end]=='`' ) i++; else i = 0;
+    if( data[end]==delim ) i++; else i = 0;
   }
   if( i<nb && end>=size ) return 0; /* no matching delimiter */
 
@@ -750,9 +766,9 @@ static size_t char_codespan(
   if( f_begin<f_end ){
     struct Blob work = BLOB_INITIALIZER;
     blob_init(&work, data+f_begin, f_end-f_begin);
-    if( !rndr->make.codespan(ob, &work, rndr->make.opaque) ) end = 0;
+    if( !rndr->make.codespan(ob, &work, nb, rndr->make.opaque) ) end = 0;
   }else{
-    if( !rndr->make.codespan(ob, 0, rndr->make.opaque) ) end = 0;
+    if( !rndr->make.codespan(ob, 0, nb, rndr->make.opaque) ) end = 0;
   }
   return end;
 }
@@ -988,10 +1004,9 @@ static size_t char_link(
   while( i<size && (data[i]==' ' || data[i]=='\t' || data[i]=='\n') ){ i++; }
 
   /* allocate temporary buffers to store content, link and title */
+  title = new_work_buffer(rndr);
   content = new_work_buffer(rndr);
   link = new_work_buffer(rndr);
-  title = new_work_buffer(rndr);
-  if( !title ) return 0;
   ret = 0; /* error if we don't get to the callback */
 
   /* inline style link */
@@ -1199,6 +1214,18 @@ static size_t prefix_code(char *data, size_t size){
   return 0;
 }
 
+/* Return the number of characters in the delimiter of a fenced code
+** block. */
+static size_t prefix_fencedcode(char *data, size_t size){
+  char c = data[0];
+  int nb;
+  if( c!='`' && c!='~' ) return 0;
+  for(nb=1; nb<size-3 && data[nb]==c; nb++){}
+  if( nb<3 ) return 0;
+  if( nb>=size-nb ) return 0;
+  return nb;
+}
+
 /* prefix_oli -- returns ordered list item prefix */
 static size_t prefix_oli(char *data, size_t size){
   size_t i = 0;
@@ -1210,7 +1237,7 @@ static size_t prefix_oli(char *data, size_t size){
   while( i<size && data[i]>='0' && data[i]<='9' ){ i++; }
 
   if( i+1>=size
-   || data[i]!='.'
+   || (data[i]!='.' && data[i]!=')')
    || (data[i+1]!=' ' && data[i+1]!='\t')
   ){
    return 0;
@@ -1284,13 +1311,12 @@ static size_t parse_blockquote(
   }
 
   if( rndr->make.blockquote ){
-    struct Blob fallback = BLOB_INITIALIZER;
-    if( out ){
+    if( !too_deep(rndr) ){
       parse_block(out, rndr, work_data, work_size);
     }else{
-      blob_init(&fallback, work_data, work_size);
+      blob_append(out, work_data, work_size);
     }
-    rndr->make.blockquote(ob, out ? out : &fallback, rndr->make.opaque);
+    rndr->make.blockquote(ob, out, rndr->make.opaque);
   }
   release_work_buffer(rndr, out);
   return end;
@@ -1308,10 +1334,14 @@ static size_t parse_paragraph(
   int level = 0;
   char *work_data = data;
   size_t work_size = 0;
-  struct Blob fallback = BLOB_INITIALIZER;
 
   while( i<size ){
-    for(end=i+1; end<size && data[end-1]!='\n'; end++);
+    char *zEnd = memchr(data+i, '\n', size-i-1);
+    end = zEnd==0 ? size : (int)(zEnd - (data-1));
+    /* The above is the same as:
+    **    for(end=i+1; end<size && data[end-1]!='\n'; end++);
+    ** "end" is left with a value such that data[end] is one byte
+    ** past the first '\n' or one byte past the end of the string */
     if( is_empty(data+i, size-i)
      || (level = is_headerline(data+i, size-i))!= 0
     ){
@@ -1330,12 +1360,8 @@ static size_t parse_paragraph(
   if( !level ){
     if( rndr->make.paragraph ){
       struct Blob *tmp = new_work_buffer(rndr);
-      if( tmp ){
-        parse_inline(tmp, rndr, work_data, work_size);
-      }else{
-        blob_init(&fallback, work_data, work_size);
-      }
-      rndr->make.paragraph(ob, tmp ? tmp : &fallback, rndr->make.opaque);
+      parse_inline(tmp, rndr, work_data, work_size);
+      rndr->make.paragraph(ob, tmp, rndr->make.opaque);
       release_work_buffer(rndr, tmp);
     }
   }else{
@@ -1348,13 +1374,9 @@ static size_t parse_paragraph(
       while( work_size && data[work_size-1]=='\n'){ work_size--; }
       if( work_size ){
         struct Blob *tmp = new_work_buffer(rndr);
-        if( tmp ){
-          parse_inline(tmp, rndr, work_data, work_size);
-        }else{
-          blob_init (&fallback, work_data, work_size);
-        }
+        parse_inline(tmp, rndr, work_data, work_size);
         if( rndr->make.paragraph ){
-          rndr->make.paragraph(ob, tmp ? tmp : &fallback, rndr->make.opaque);
+          rndr->make.paragraph(ob, tmp, rndr->make.opaque);
         }
         release_work_buffer(rndr, tmp);
         work_data += beg;
@@ -1366,13 +1388,8 @@ static size_t parse_paragraph(
 
     if( rndr->make.header ){
       struct Blob *span = new_work_buffer(rndr);
-      if( span ){
-        parse_inline(span, rndr, work_data, work_size);
-        rndr->make.header(ob, span, level, rndr->make.opaque);
-      }else{
-        blob_init(&fallback, work_data, work_size);
-        rndr->make.header(ob, &fallback, level, rndr->make.opaque);
-      }
+      parse_inline(span, rndr, work_data, work_size);
+      rndr->make.header(ob, span, level, rndr->make.opaque);
       release_work_buffer(rndr, span);
     }
   }
@@ -1389,11 +1406,15 @@ static size_t parse_blockcode(
 ){
   size_t beg, end, pre;
   struct Blob *work = new_work_buffer(rndr);
-  if( !work ) work = ob;
 
   beg = 0;
   while( beg<size ){
-    for(end=beg+1; end<size && data[end-1]!='\n'; end++);
+    char *zEnd = memchr(data+beg, '\n', size-beg-1);
+    end = zEnd==0 ? size : (int)(zEnd - (data-1));
+    /* The above is the same as:
+    **   for(end=beg+1; end<size && data[end-1]!='\n'; end++);
+    ** "end" is left with a value such that data[end] is one byte
+    ** past the first \n or past then end of the string. */
     pre = prefix_code(data+beg, end-beg);
     if( pre ){
       beg += pre; /* skipping prefix */
@@ -1404,7 +1425,7 @@ static size_t parse_blockcode(
     if( beg<end ){
       /* verbatim copy to the working buffer, escaping entities */
       if( is_empty(data + beg, end - beg) ){
-        blob_append(work, "\n", 1);
+        blob_append_char(work, '\n');
       }else{
         blob_append(work, data+beg, end-beg);
       }
@@ -1415,7 +1436,7 @@ static size_t parse_blockcode(
   end = blob_size(work);
   while( end>0 && blob_buffer(work)[end-1]=='\n' ){ end--; }
   work->nUsed = end;
-  blob_append(work, "\n", 1);
+  blob_append_char(work, '\n');
 
   if( work!=ob ){
     if( rndr->make.blockcode ){
@@ -1436,7 +1457,6 @@ static size_t parse_listitem(
   size_t size,
   int *flags
 ){
-  struct Blob fallback = BLOB_INITIALIZER;
   struct Blob *work = 0, *inter = 0;
   size_t beg = 0, end, pre, sublist = 0, orgpre = 0, i;
   int in_empty = 0, has_inside_empty = 0;
@@ -1461,7 +1481,6 @@ static size_t parse_listitem(
   /* getting working buffers */
   work = new_work_buffer(rndr);
   inter = new_work_buffer(rndr);
-  if( !work ) work = &fallback;
 
   /* putting the first line into the working buffer */
   blob_append(work, data+beg, end-beg);
@@ -1511,7 +1530,7 @@ static size_t parse_listitem(
         *flags |= MKD_LI_END;
         break;
     }else if( in_empty ){
-      blob_append(work, "\n", 1);
+      blob_append_char(work, '\n');
       has_inside_empty = 1;
     }
     in_empty = 0;
@@ -1526,8 +1545,7 @@ static size_t parse_listitem(
     if( rndr->make.listitem ){
       rndr->make.listitem(ob, work, *flags, rndr->make.opaque);
     }
-    if( work!=&fallback ) release_work_buffer(rndr, work);
-    blob_reset(&fallback);
+    release_work_buffer(rndr, work);
     return beg;
   }
 
@@ -1562,8 +1580,7 @@ static size_t parse_listitem(
     rndr->make.listitem(ob, inter, *flags, rndr->make.opaque);
   }
   release_work_buffer(rndr, inter);
-  if( work!=&fallback ) release_work_buffer(rndr, work);
-  blob_reset(&fallback);
+  release_work_buffer(rndr, work);
   return beg;
 }
 
@@ -1576,10 +1593,8 @@ static size_t parse_list(
   size_t size,
   int flags
 ){
-  struct Blob fallback = BLOB_INITIALIZER;
   struct Blob *work = new_work_buffer(rndr);
   size_t i = 0, j;
-  if( !work ) work = &fallback;
 
   while( i<size ){
     j = parse_listitem(work, rndr, data+i, size-i, &flags);
@@ -1588,8 +1603,7 @@ static size_t parse_list(
   }
 
   if( rndr->make.list ) rndr->make.list(ob, work, flags, rndr->make.opaque);
-  if( work!=&fallback ) release_work_buffer(rndr, work);
-  blob_reset(&fallback);
+  release_work_buffer(rndr, work);
   return i;
 }
 
@@ -1619,15 +1633,9 @@ static size_t parse_atxheader(
 
   span_size = end-span_beg;
   if( rndr->make.header ){
-    struct Blob fallback = BLOB_INITIALIZER;
     struct Blob *span = new_work_buffer(rndr);
-
-    if( span ){
-      parse_inline(span, rndr, data+span_beg, span_size);
-    }else{
-      blob_init(&fallback, data+span_beg, span_size);
-    }
-    rndr->make.header(ob, span ? span : &fallback, level, rndr->make.opaque);
+    parse_inline(span, rndr, data+span_beg, span_size);
+    rndr->make.header(ob, span, level, rndr->make.opaque);
     release_work_buffer(rndr, span);
   }
   return skip;
@@ -1788,15 +1796,9 @@ static void parse_table_cell(
   size_t size,         /* input text size */
   int flags            /* table flags */
 ){
-  struct Blob fallback = BLOB_INITIALIZER;
   struct Blob *span = new_work_buffer(rndr);
-
-  if( span ){
-    parse_inline(span, rndr, data, size);
-  }else{
-    blob_init(&fallback, data, size);
-  }
-  rndr->make.table_cell(ob, span ? span : &fallback, flags, rndr->make.opaque);
+  parse_inline(span, rndr, data, size);
+  rndr->make.table_cell(ob, span, flags, rndr->make.opaque);
   release_work_buffer(rndr, span);
 }
 
@@ -1858,19 +1860,15 @@ static size_t parse_table_row(
     if( align==0 && aligns && col<align_size ) align = aligns[col];
 
     /* render cells */
-    if( cells ) parse_table_cell(cells, rndr, data+beg, end-beg, align|flags);
+    if( cells && end>beg ){
+      parse_table_cell(cells, rndr, data+beg, end-beg, align|flags);
+    }
 
     col++;
   }
 
   /* render the whole row and clean up */
-  if( cells ){
-    rndr->make.table_row(ob, cells, flags, rndr->make.opaque);
-  }else{
-    struct Blob fallback = BLOB_INITIALIZER;
-    blob_init(&fallback, data, total ? total : size);
-    rndr->make.table_row(ob, &fallback, flags, rndr->make.opaque);
-  }
+  rndr->make.table_row(ob, cells, flags, rndr->make.opaque);
   release_work_buffer(rndr, cells);
   return total ? total : size;
 }
@@ -1886,10 +1884,8 @@ static size_t parse_table(
   size_t i = 0, head_end, col;
   size_t align_size = 0;
   int *aligns = 0;
-  struct Blob fallback = BLOB_INITIALIZER;
   struct Blob *head = 0;
   struct Blob *rows = new_work_buffer(rndr);
-  if( !rows ) rows = &fallback;
 
   /* skip the first (presumably header) line */
   while( i<size && data[i]!='\n' ){ i++; }
@@ -1899,7 +1895,7 @@ static size_t parse_table(
   if( i>=size ){
     parse_table_row(rows, rndr, data, size, 0, 0, 0);
     rndr->make.table(ob, 0, rows, rndr->make.opaque);
-    if( rows!=&fallback ) release_work_buffer(rndr, rows);
+    release_work_buffer(rndr, rows);
     return i;
   }
 
@@ -1923,9 +1919,7 @@ static size_t parse_table(
 
     /* render the header row */
     head = new_work_buffer(rndr);
-    if( head ){
-      parse_table_row(head, rndr, data, head_end, 0, 0, MKD_CELL_HEAD);
-    }
+    parse_table_row(head, rndr, data, head_end, 0, 0, MKD_CELL_HEAD);
 
     /* parse alignments if provided */
     if( col && (aligns=fossil_malloc(align_size * sizeof *aligns))!=0 ){
@@ -1968,8 +1962,8 @@ static size_t parse_table(
   rndr->make.table(ob, head, rows, rndr->make.opaque);
 
   /* cleanup */
-  if( head ) release_work_buffer(rndr, head);
-  if( rows!=&fallback ) release_work_buffer(rndr, rows);
+  release_work_buffer(rndr, head);
+  release_work_buffer(rndr, rows);
   fossil_free(aligns);
   return i;
 }
@@ -2015,6 +2009,10 @@ static void parse_block(
       beg += parse_list(ob, rndr, txt_data, end, MKD_LIST_ORDERED);
     }else if( has_table && is_tableline(txt_data, end) ){
       beg += parse_table(ob, rndr, txt_data, end);
+    }else if( prefix_fencedcode(txt_data, end) 
+             && (i = char_codespan(ob, rndr, txt_data, 0, end))!=0
+    ){
+      beg += i;
     }else{
       beg += parse_paragraph(ob, rndr, txt_data, end);
     }
@@ -2163,19 +2161,17 @@ void markdown(
   const struct mkd_renderer *rndrer  /* renderer descriptor (callbacks) */
 ){
   struct link_ref *lr;
-  struct Blob text = BLOB_INITIALIZER;
   size_t i, beg, end = 0;
   struct render rndr;
   char *ib_data;
+  Blob text = BLOB_INITIALIZER;
 
   /* filling the render structure */
   if( !rndrer ) return;
   rndr.make = *rndrer;
-  if( rndr.make.max_work_stack<1 ) rndr.make.max_work_stack = 1;
-  rndr.work_active = 0;
-  rndr.work = fossil_malloc(rndr.make.max_work_stack * sizeof *rndr.work);
-  for(i=0; i<rndr.make.max_work_stack; i++) rndr.work[i] = text;
-  rndr.refs = text;
+  rndr.nBlobCache = 0;
+  rndr.iDepth = 0;
+  rndr.refs = empty_blob;
   for(i=0; i<256; i++) rndr.active_char[i] = 0;
   if( (rndr.make.emphasis
     || rndr.make.double_emphasis
@@ -2211,7 +2207,7 @@ void markdown(
         if( ib_data[end]=='\n'
          || (end+1<blob_size(ib) && ib_data[end+1]!='\n')
         ){
-          blob_append(&text, "\n", 1);
+          blob_append_char(&text, '\n');
         }
         end += 1;
       }
@@ -2233,6 +2229,7 @@ void markdown(
   if( rndr.make.epilog ) rndr.make.epilog(ob, rndr.make.opaque);
 
   /* clean-up */
+  assert( rndr.iDepth==0 );
   blob_reset(&text);
   lr = (struct link_ref *)blob_buffer(&rndr.refs);
   end = blob_size(&rndr.refs)/sizeof(struct link_ref);
@@ -2242,6 +2239,7 @@ void markdown(
     blob_reset(&lr[i].title);
   }
   blob_reset(&rndr.refs);
-  blobarray_zero(rndr.work, rndr.make.max_work_stack);
-  fossil_free(rndr.work);
+  for(i=0; i<rndr.nBlobCache; i++){
+    fossil_free(rndr.aBlobCache[i]);
+  }
 }
