@@ -299,6 +299,9 @@ void user_edit(void){
     if( P("verifydelete") ){
       /* Verified delete user request */
       db_multi_exec("DELETE FROM user WHERE uid=%d", uid);
+      moderation_disapprove_for_missing_users();
+      admin_log("Deleted user [%s] (uid %d).",
+                PD("login","???")/*safe-for-%s*/, uid);
       cgi_redirect(cgi_referer("setup_ulist"));
       return;
     }
@@ -336,7 +339,7 @@ void user_edit(void){
     zNm[2] = 0;
     for(i=0, c='a'; c<='z'; c++){
       zNm[1] = c;
-      a[c&0x7f] = (c!='s' || g.perm.Setup) && P(zNm)!=0;
+      a[c&0x7f] = ((c!='s' && c!='y') || g.perm.Setup) && P(zNm)!=0;
       if( a[c&0x7f] ) zCap[i++] = c;
     }
     for(c='0'; c<='9'; c++){
