@@ -538,7 +538,7 @@ static int stash_get_id(const char *zStashId){
 **  fossil stash pop
 **  fossil stash apply ?STASHID?
 **
-**     Apply STASHID or the most recently create stash to the current
+**     Apply STASHID or the most recently created stash to the current
 **     working checkout.  The "pop" command deletes that changeset from
 **     the stash after applying it but the "apply" command retains the
 **     changeset.
@@ -591,6 +591,9 @@ void stash_cmd(void){
   }
   nCmd = strlen(zCmd);
   if( memcmp(zCmd, "save", nCmd)==0 ){
+    if( unsaved_changes(0)==0 ){
+      fossil_fatal("nothing to stash");
+    }
     stashid = stash_create();
     undo_disable();
     if( g.argc>=2 ){
@@ -610,8 +613,12 @@ void stash_cmd(void){
       g.argc = nFile+2;
       if( nFile==0 ) return;
     }
+    /* Make sure the stash has committed before running the revert, so that
+    ** we have a copy of the changes before deleting them. */
+    db_commit_transaction();
     g.argv[1] = "revert";
     revert_cmd();
+    return;
   }else
   if( memcmp(zCmd, "snapshot", nCmd)==0 ){
     stash_create();
