@@ -898,7 +898,7 @@ void tkttimeline_page(void){
   }
   db_prepare(&q, "%z", zSQL/*safe-for-%s*/);
   www_print_timeline(&q, TIMELINE_ARTID|TIMELINE_DISJOINT|TIMELINE_GRAPH,
-                     0, 0, 0, 0);
+                     0, 0, 0, 0, 0, 0);
   db_finalize(&q);
   style_footer();
 }
